{ Based on
  https://lazplanet.blogspot.com/2013/12/create-rich-text-editor-for-yourself.html
}

unit Main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls, ExtCtrls,
  StdCtrls, Menus, RichMemo;

type
  TRichMemoParagraphAttributes = record
    Alignment: TParaAlignment;
    Numbering: TParaNumbering;
    HeadIndent: Double;  // pts
  end;

  { TForm1 }

  TForm1 = class(TForm)
    btnBkColor: TToolButton;
    cmbFont: TComboBox;
    cmbFontSize: TComboBox;
    ColorDialog: TColorDialog;
    ImageList: TImageList;
    Label1: TLabel;
    Label2: TLabel;
    mnuUpRoman: TMenuItem;
    mnuListBullet: TMenuItem;
    mnuListLowLetter: TMenuItem;
    mnuListLowRoman: TMenuItem;
    mnuListUpLetter: TMenuItem;
    mnuListNone: TMenuItem;
    mnuListNumber: TMenuItem;
    OpenDialog: TOpenDialog;
    Panel1: TPanel;
    Panel2: TPanel;
    ListDropDown: TPopupMenu;
    RichMemo: TRichMemo;
    SaveDialog: TSaveDialog;
    StatusBar1: TStatusBar;
    ToolBar1: TToolBar;
    btnNew: TToolButton;
    btnOpen: TToolButton;
    btnSaveAs: TToolButton;
    ToolButton1: TToolButton;
    btnBold: TToolButton;
    btnItalic: TToolButton;
    btnUnderline: TToolButton;
    ToolButton2: TToolButton;
    btnFontColor: TToolButton;
    btnSubscript: TToolButton;
    btnSuperscript: TToolButton;
    ToolButton3: TToolButton;
    btnAlignLeft: TToolButton;
    btnAlignCenter: TToolButton;
    btnAlignRight: TToolButton;
    btnAlignJustified: TToolButton;
    btnIndent: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    btnOutdent: TToolButton;
    procedure btnAlignCenterClick(Sender: TObject);
    procedure btnAlignJustifiedClick(Sender: TObject);
    procedure btnAlignLeftClick(Sender: TObject);
    procedure btnAlignRightClick(Sender: TObject);
    procedure btnBkColorClick(Sender: TObject);
    procedure btnBoldClick(Sender: TObject);
    procedure btnFontColorClick(Sender: TObject);
    procedure btnIndentClick(Sender: TObject);
    procedure btnItalicClick(Sender: TObject);
    procedure btnNewClick(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure btnOutdentClick(Sender: TObject);
    procedure btnSaveAsClick(Sender: TObject);
    procedure btnSubscriptClick(Sender: TObject);
    procedure btnSuperscriptClick(Sender: TObject);
    procedure btnUnderlineClick(Sender: TObject);
    procedure mnuListBulletClick(Sender: TObject);
    procedure cmbFontSelect(Sender: TObject);
    procedure cmbFontSizeSelect(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure RichMemoChange(Sender: TObject);
    procedure RichMemoMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure RichMemoSelectionChange(Sender: TObject);
  private
    FSaved: Boolean;
    FFileName: String;
    FFilePath: String;
    FSelFontFormat: TFontParams;
    FSelParaFormat: TRichMemoParagraphAttributes;
    procedure PrepareToolbar();
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

uses
  uRtf2Html;

const
  DEFAULT_INDENT = 5;  // mm

{ Converts millimeters to pixels }
function mm2px(mm: Double): Integer;
begin
  Result := round(ScreenInfo.PixelsPerInchX * (mm / 25.4));
end;

{ TForm1 }

procedure TForm1.btnAlignLeftClick(Sender: TObject);
begin
  FSelParaFormat.Alignment := paLeft;
  RichMemo.SetParaAlignment(RichMemo.SelStart, RichMemo.SelLength, FSelParaFormat.Alignment);
end;

procedure TForm1.btnAlignRightClick(Sender: TObject);
begin
  FSelParaFormat.Alignment := paRight;
  RichMemo.SetParaAlignment(RichMemo.SelStart, RichMemo.SelLength, FSelParaFormat.Alignment);
end;

procedure TForm1.btnAlignCenterClick(Sender: TObject);
begin
  FSelParaFormat.Alignment := paCenter;
  RichMemo.SetParaAlignment(RichMemo.SelStart, RichMemo.SelLength, FSelParaFormat.Alignment);
end;

procedure TForm1.btnAlignJustifiedClick(Sender: TObject);
begin
  FSelParaFormat.Alignment := paJustify;
  RichMemo.SetParaAlignment(RichMemo.SelStart, RichMemo.SelLength, FSelParaFormat.Alignment);
end;

procedure TForm1.btnBkColorClick(Sender: TObject);
begin
  if not FSelFontFormat.HasBkClr then
    ColorDialog.Color := clWhite
  else
    ColorDialog.Color := FSelFontFormat.BkColor;
  if ColorDialog.Execute then
  begin
    FSelFontFormat.BkColor := ColorDialog.Color;
    FSelFontFormat.HasBkClr := true;
    RichMemo.SetTextAttributes(RichMemo.SelStart, RichMemo.SelLength, FSelFontFormat);
  end;
end;

procedure TForm1.btnBoldClick(Sender: TObject);
begin
  if not (fsBold in FSelFontFormat.Style) then
    FSelFontFormat.Style := FSelFontFormat.Style + [fsBold]
  else
    FSelFontFormat.Style := FSelFontFormat.Style - [fsBold];
  RichMemo.SetTextAttributes(RichMemo.SelStart, RichMemo.SelLength, FSelFontFormat);
end;

procedure TForm1.btnFontColorClick(Sender: TObject);
begin
  ColorDialog.Color := FSelFontFormat.Color;
  if ColorDialog.Execute then
  begin
    FSelFontFormat.Color := ColorDialog.Color;
    RichMemo.SetTextAttributes(RichMemo.SelStart, RichMemo.SelLength, FSelFontFormat);
  end;
end;

procedure TForm1.btnIndentClick(Sender: TObject);
var
  pm: TParaMetric;
begin
  RichMemo.GetParaMetric(RichMemo.SelStart, pm);
  pm.HeadIndent := pm.HeadIndent + mm2px(DEFAULT_INDENT);
  pm.FirstLine := pm.HeadIndent;
  RichMemo.SetParaMetric(RichMemo.SelStart, RichMemo.SelLength, pm);
end;

procedure TForm1.btnItalicClick(Sender: TObject);
begin
  if not (fsItalic in FSelFontFormat.Style) then
    FSelFontFormat.Style := FSelFontFormat.Style + [fsItalic]
  else
    FSelFontFormat.Style := FSelFontFormat.Style - [fsItalic];
  RichMemo.SetTextAttributes(RichMemo.SelStart, RichMemo.SelLength, FSelFontFormat);
end;

procedure TForm1.btnNewClick(Sender: TObject);
begin
  RichMemo.Clear;
  FSaved := true;
  FFilename := 'Untitled.rtf';
  FFilepath := '';
  Caption := FFilename;
end;

procedure TForm1.btnOpenClick(Sender: TObject);
var
  fs: TFileStream = nil;
begin
  if OpenDialog.Execute then begin
    fs := TFileStream.Create(OpenDialog.FileName, fmOpenRead or fmShareDenyNone);
    try
      RichMemo.LoadRichText(fs);
      FSaved := True;  // since we opened a saved file
      FFilename := ExtractFileName(OpenDialog.FileName);
      FFilepath := ExtractFilePath(OpenDialog.FileName);
      Caption := FFilename;
    finally
      fs.Free;
    end;
  end;
end;

procedure TForm1.btnOutdentClick(Sender: TObject);
var
  pm: TParaMetric;
begin
  RichMemo.GetParaMetric(RichMemo.SelStart, pm);
  pm.HeadIndent := pm.HeadIndent - mm2px(DEFAULT_INDENT);  // 10 mm
  if pm.HeadIndent < 0 then pm.HeadIndent := 0;
  pm.FirstLine := pm.HeadIndent;
  RichMemo.SetParaMetric(RichMemo.SelStart, RichMemo.SelLength, pm);
end;

procedure TForm1.btnSaveAsClick(Sender: TObject);
var
  fs: TFileStream = nil;
  rtf2html: TRtf2HtmlConverter;
  stream: TMemoryStream;
begin
  if SaveDialog.Execute then begin
    Screen.BeginWaitCursor;
    try
      if SaveDialog.FilterIndex = 1 then   // RTF
      begin
        fs := TFileStream.Create(SaveDialog.FileName, fmCreate);
        try
          RichMemo.SaveRichText(fs);
          FSaved := True;
          FFilename := ExtractFileName(SaveDialog.FileName);
          FFilepath := ExtractFilePath(SaveDialog.FileName);
          Caption := FFilename;
        finally
          fs.Free;
        end;
      end else
      if SaveDialog.FilterIndex = 2 then    // HTML
      begin
        stream := TMemoryStream.Create;
        try
          RichMemo.SaveRichText(stream);
          stream.Position := 0;
          rtf2html := TRtf2HtmlConverter.Create(stream);
          try
            rtf2html.ConvertToHtml(SaveDialog.FileName);
          finally
            rtf2html.Free;
          end;
        finally
          stream.Free;
        end;
      end;
    finally
      Screen.EndWaitCursor;
    end;
  end;
end;

procedure TForm1.btnSubscriptClick(Sender: TObject);
begin
  if FSelFontFormat.VScriptPos = vpSubScript then
    FSelFontFormat.VScriptPos := vpNormal
  else
    FSelFontFormat.VScriptPos := vpSubScript;
  RichMemo.SetTextAttributes(RichMemo.SelStart, RichMemo.SelLength, FSelFontFormat);
end;

procedure TForm1.btnSuperscriptClick(Sender: TObject);
begin
  if FSelFontFormat.VScriptPos = vpSuperScript then
    FSelFontFormat.VScriptPos := vpNormal
  else
    FSelFontFormat.VScriptPos := vpSuperScript;
  RichMemo.SetTextAttributes(RichMemo.SelStart, RichMemo.SelLength, FSelFontFormat);
end;

procedure TForm1.btnUnderlineClick(Sender: TObject);
begin
  if not (fsUnderline in FSelFontFormat.Style) then
    FSelFontFormat.Style := FSelFontFormat.Style + [fsUnderline]
  else
    FSelFontFormat.Style := FSelFontFormat.Style - [fsUnderline];
  RichMemo.SetTextAttributes(RichMemo.SelStart, RichMemo.SelLength, FSelFontFormat);
end;

procedure TForm1.mnuListBulletClick(Sender: TObject);
var
  pn: TParaNumbering;
begin
  RichMemo.GetParaNumbering(RichMemo.SelStart, pn);
  pn.Style := TParaNumStyle(TMenuItem(Sender).Tag);
  pn.Indent := mm2px(DEFAULT_INDENT);
  pn.NumberStart := 1;
  RichMemo.SetParaNumbering(RichMemo.SelStart, RichMemo.SelLength, pn);
end;

procedure TForm1.cmbFontSelect(Sender: TObject);
begin
  FSelFontFormat.Name := cmbFont.Text;
  RichMemo.SetTextAttributes(RichMemo.SelStart, RichMemo.SelLength, FSelFontFormat);
  RichMemo.SetFocus;
  RichMemo.Invalidate;
end;

procedure TForm1.cmbFontSizeSelect(Sender: TObject);
begin
  FSelFontFormat.Size := StrToInt(cmbFontSize.Text);
  RichMemo.SetTextAttributes(RichMemo.SelStart, RichMemo.SelLength, FSelFontFormat);
  RichMemo.SetFocus;
  RichMemo.Invalidate;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  Response: Integer;
begin
  if not FSaved then
  begin
    response := MessageDlg('Save?', 'Do you wish to Save?', mtConfirmation, mbYesNoCancel, 0);
    if response = mrYes then
    begin
      btnSaveAsClick(Sender); // we save it
      CanClose := True; // we let it close
    end else
    if response = mrNo then begin
      CanClose := True; // we let it close (but do not save)
    end else
    begin
      CanClose := false; // we don't need to close
    end;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  RichMemo.GetTextAttributes(0, FSelFontFormat);
  RichMemo.GetParaAlignment(0, FSelParaFormat.Alignment);
  RichMemo.GetParaNumbering(0, FSelParaFormat.Numbering);

  cmbFont.Items.Assign(Screen.Fonts);
  cmbFont.ItemIndex := Screen.Fonts.IndexOf(Screen.SystemFont.Name);

  btnNewClick(Sender);
  ActiveControl := RichMemo;
end;

procedure TForm1.PrepareToolbar();
begin
  cmbFont.Caption := FSelFontFormat.Name;
  cmbFontSize.Caption := IntToStr(FSelFontFormat.Size);

  btnBold.Down := (fsBold in FSelFontFormat.Style);
  btnItalic.Down := (fsItalic in FSelFontFormat.Style);
  btnUnderline.Down := (fsUnderline in FSelFontFormat.Style);
  btnSubscript.Down := (FSelFontFormat.VScriptPos = vpSubscript);
  btnSuperscript.Down := (FSelFontFormat.VScriptPos = vpSuperscript);

  case FSelParaFormat.Alignment of
    paLeft: btnAlignLeft.Down := true;
    paCenter: btnAlignCenter.Down := true;
    paRight: btnAlignRight.Down := true;
    paJustify: btnAlignJustified.Down := true;
  end;
end;

procedure TForm1.RichMemoChange(Sender: TObject);
begin
  FSaved := false;
end;

procedure TForm1.RichMemoMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  RichMemo.GetTextAttributes(RichMemo.SelStart, FSelFontFormat);
  RichMemo.GetParaAlignment(0, FSelParaFormat.Alignment);
  RichMemo.GetParaNumbering(0, FSelParaFormat.Numbering);

  PrepareToolbar;
end;

procedure TForm1.RichMemoSelectionChange(Sender: TObject);
begin
  Statusbar1.SimpleText := Format('SelStart: %d, SelLength: %d', [RichMemo.SelStart, RichMemo.SelLength]);
end;

end.

